/*
* Copyright (c) 2008 Nokia Corporation and/or its subsidiary(-ies). 
* All rights reserved.
* This component and the accompanying materials are made available
* under the terms of "Eclipse Public License v1.0"
* which accompanies this distribution, and is available
* at the URL "http://www.eclipse.org/legal/epl-v10.html".
*
* Initial Contributors:
* Nokia Corporation - initial contribution.
*
* Contributors:
*
* Description:   Videolist selection dialog 
* 
*/

#ifndef VIDEOLISTSELECTIONDIALOG_H
#define VIDEOLISTSELECTIONDIALOG_H

#include <hbselectiondialog.h>
#include <qset.h>
#include <mpxitemid.h>
#include <qpointer.h>

class QGraphicsItem;
class QTimer;
class VideoProxyModelGeneric;
class VideoCollectionUiLoader;

class VideoListSelectionDialog: public HbSelectionDialog
{
    /**
     * definition required for emitting / connecting signals 
     */
    Q_OBJECT
    
    /**
     * disable copy-constructor and assignment operator
     */
    Q_DISABLE_COPY(VideoListSelectionDialog) 

    
public: 
    
    enum TSelectionFunction
    {
        ENoFunction = -1,
        EDeleteVideos,
        EAddToCollection,        
        ERemoveFromCollection,
        ECreateCollection,
        ESelectCollection
    };
    
    /**
     * Constructor
     *
     * @param parent item's parent component
     */
    VideoListSelectionDialog(QGraphicsItem *parent=0);

    /**
     * Destructor
     */
    ~VideoListSelectionDialog();
   
    /**
     * Method setups provided title and videolist content widget.
     * Prepares dialog for showing.
     * If either provided title is empty or widget is NULL, dialog
     * is in invalid state and cannot be shown using exec.
     *
     * @param type selection dialog function typy
     * @param activeItem id of item that the selection concerns
     */
    void setupContent(VideoListSelectionDialog::TSelectionFunction type, TMPXItemId activeItem = TMPXItemId::InvalidId());

public slots:

    /**
     * Shows modal dialog build from the provided Videolistwidget.
     *
     */
    void exec();
    
    /**
     * Signaled by HbInputDialog when it's closed.
     */
    void newAlbumNameDialogFinished(HbAction *action);    
    
private slots:
    
    /**
     * disables or enables popup's primary action in case selection changes
     */
    void selectionChangedSlot();

    /**
     * Handles model ready signal from model.
     */
    void modelReadySlot();
    
    /**
     * Handles album list ready signal from model.
     */
    void albumListReadySlot();
    
    /**
     * Primary action triggered signal slot.
     * 
     */
    void primaryActionTriggeredSlot();
    
    /**
     * called when dialog has been finished
     * 
     * @param action causing the finished
     */
    void finishedSlot(HbAction *action);

    /**
     * Signaled when view scrolling starts, pauses thumbnail creation.
     *
     */
    void scrollingStartedSlot();

    /**
     * Signaled when view scrolling ends, initiates thumbnail fetching
     * at index of first visible item and enables creation of thumbnails.
     *
     */
    void scrollingEndedSlot();

    /**
     * Signaled when view scroll position changes, initiates timer to
     * fetch thumbnails at index of first visible item.
     *
     * @param newPosition scroll position
     *
     */
    void scrollPositionChangedSlot(const QPointF &newPosition);

    /**
     * Signaled when scroll position timer triggers. Starts fetching thumbnails
     * for visible items.
     *
     */
    void scrollPositionTimerSlot();

    /**
     * Fetches thumbnails for the visible items.
     *
     */
    void fetchThumbnailsForVisibleItems();

private:
    
    /**
     * Method initializes dialog by getting all ui components and 
     * checking if they are valid
     * 
     * @return bool true if inisializaion ok
     */
    bool initDialog();

    /**
     * Method initializes list by setting the model
     *
     * @param model to be set
     *
     * @return bool true if inisializaion ok
     */
    bool initList(VideoProxyModelGeneric *model);

    /**
     * Method activated dialog based on value in mTypeOfSelection
     *
     * @param changeMode true if dialog mode is changed from multiselection to noselection while visible
     *
     */
    void activateSelection(bool changeMode = false);

    /**
     * connects all required signals into appropriate slots 
     * for selection mode
     */
    void connectSignals();
    
    /**
     * disconnects all signals 
     */
    void disconnectSignals();
    
    /**
     * Gets selected item's name from appropriate model.
     * 
     * @return QString item's name
     */
    QString getSelectedName();
    
    /**
     * Finalizes the operation.
     */
    void finalize(QString albumName = "");
    
private:
   
    /**
     * type for defining selection functionality
     */
    VideoListSelectionDialog::TSelectionFunction mTypeOfSelection;

    /**
     * Selected video items
     */
    QSet<TMPXItemId> mSelectedVideos;
    
    /**
     * selected single items album id
     */
    TMPXItemId mSelectedAlbumId;
    
    /**
     * Text for a new album.
     */
    QString mNewAlbumText;
    
    /**
     * Video list model
     */
    VideoProxyModelGeneric *mModel;
    
    /**
     * Boolean to know if the model is ready or not. 
     */
    bool mModelReady;
    
    /**
     * Boolean to know if album list is ready or not.
     */
    bool mAlbumListReady;

    /**
     * Boolean to know if listview has been initialised or not.
     */
    bool mListInitialised;

    /**
     * Action to accept the selection dialog. 
     */
    QPointer<HbAction> mPrimaryAction;

    /**
     * Action to cancel the selection dialog.
     */
    QPointer<HbAction> mSecondaryAction;

    /**
     * Timer used to report thumbnail fetches class index of
     * first visible item when view is scrolling.
     */
    QPointer<QTimer> mScrollPositionTimer;
};

#endif  //VIDEOLISTSELECTIONDIALOG_H
